package arc.ex;

import arc.ex.ControlModule.CmException;

public class CM01 extends ControlModule{

	// R}h
	final static byte GET_CONFIG = 50;
	final static byte CLEAR_DATA = (byte)51;
	final static byte EXCHANGE 	 = (byte)150;

	// ̓{[hID
	final static byte ENC_BOARD 	= 0;
	final static byte SENSOR_BOARD 	= 1;

	// o̓{[hID
	final static byte PWM_BOARD 	= 0;
	final static byte DA_BOARD 		= 1;

	final static byte PIO_BOARD 	= 2;

	class InputEnc
	{
		int encoder;
		short[] sensor = new short[5];
	};

	class InputSensor
	{
		short[] sensor = new short[8];
	}

	class OutputPwm
	{
		short pwm1;
		short pwm2;
	}
	
	class OutputDa
	{
		short da;
	}

	class Pio
	{
		short data;
		short dir;
	}

	byte boardID_PA;
	byte boardID_PB;
	short inputDataSize;
	short outputDataSize;

	short inputDataSize_PA;
	short outputDataSize_PA;
	short inputDataSize_PB;
	short outputDataSize_PB;

	InputEnc encBoard;
	InputSensor sensorBoard;
	OutputPwm pwmBoard;
	OutputDa daBoard;
	Pio pioBoardA;
	Pio pioBoardB;
	
	/* Constructor */
	public CM01()
	{
		name = "CM01";

		encBoard = new InputEnc();
		sensorBoard = new InputSensor();
		pwmBoard = new OutputPwm();
		daBoard = new OutputDa();
		pioBoardA = new Pio();
		pioBoardB = new Pio();
		
		checkSumEnable = true;

		pioBoardA.dir = (short)0xffff;
		pioBoardA.data = 0;
		pioBoardB.dir = (short)0xffff;
		pioBoardB.data = 0;
		
		boardID_PA = 0;
		boardID_PB = 0;
		inputDataSize = 0;
		outputDataSize = 0;

	}
	
	@Override
	void open( ControlModule parent, int port, int id ) throws CmException
	{
		super.open( parent, port, id );
		getConfig();
	}

	void getConfig() throws CmException 
	{
		command = GET_CONFIG;
		txBuffer.put(command);

		transceive( 6, true );

		boardID_PA = rxBuffer.get();
		boardID_PB = rxBuffer.get();
		inputDataSize = rxBuffer.getShort();
		outputDataSize = rxBuffer.getShort();
		
	}

	void clearData() throws CmException
	{
		command = CLEAR_DATA;
		txBuffer.put(command);
		
		transceive( 0, true );
	}

	void exchange( boolean rightNow ) throws CmException
	{
		command = EXCHANGE;
		txBuffer.put(command);
		
		if( boardID_PA == PIO_BOARD ){
			txBuffer.putShort(pioBoardA.data);
			txBuffer.putShort(pioBoardA.dir);
		}
		switch( boardID_PB ){
		case PWM_BOARD:
			txBuffer.putShort(pwmBoard.pwm1);
			txBuffer.putShort(pwmBoard.pwm2);
			break; // PWM board
		case DA_BOARD:
			txBuffer.putShort(daBoard.da);
			break; // DA board
		case PIO_BOARD:
			txBuffer.putShort(pioBoardB.data);
			txBuffer.putShort(pioBoardB.dir);
			break; // PIO board	
		}
		
		transceive( inputDataSize, rightNow );

	}

	@Override
	void receive( int rxSize ) throws CmException
	{
		super.receive( rxSize );
		
		if( command == EXCHANGE ){
			int i;

			switch( boardID_PA){
			case ENC_BOARD:
				encBoard.encoder = rxBuffer.getInt();
				for(i=0;i<5;i++) encBoard.sensor[i] = rxBuffer.getShort();
			break;
			case SENSOR_BOARD:
				for(i=0;i<8;i++) sensorBoard.sensor[i] = rxBuffer.getShort();
				break;
			case PIO_BOARD:
				pioBoardA.data = rxBuffer.getShort();
				break;
			}
			if( boardID_PB == PIO_BOARD ){
				pioBoardB.data = rxBuffer.getShort();
			}
		}

	}
}